import os
import asyncio

from configparser import ConfigParser, ExtendedInterpolation

from aio.core.exceptions import MissingConfiguration


def find_config(app_dir=None):
    if not app_dir:
        app_dir = os.getcwd()
    config = os.path.join(app_dir, 'aio.conf')
    if os.path.exists(config):
        return config
    config = os.path.join(app_dir, 'etc', 'aio.conf')
    if os.path.exists(config):
        return config
    config = os.path.join(app_dir, '/etc', 'aio.conf')
    if os.path.exists(config):
        return config
    raise MissingConfiguration('no configuration file found')


@asyncio.coroutine
def parse_config(config=None, app_dir=None, config_dict=None, config_string=None):
    parser = ConfigParser(interpolation=ExtendedInterpolation())
    if config_dict:
        parser.read_dict(config_dict)
    elif config_string:
        parser.read_string(config_string)
    else:
        parser.read_file(open(config or find_config(app_dir)))
    return parser



import os
import unittest

from aio.testing import aiotest
from aio.config import parse_config, find_config

TEST_DIR = os.path.dirname(__file__)

CONFIG_STRING = """
[section1]
bar: 1
foo: 2

[section2]
bar: 3
foo: 4
"""


class AioConfigParseTestCase(unittest.TestCase):

    @aiotest
    def test_parse_config(self):
        """
        this test should find and parse "aio.conf" in the app_dir
        """
        app_dir = os.path.join(
            TEST_DIR, "resources")
        config = yield from parse_config(app_dir=app_dir)
        self.assertEqual(config.sections(), ["section1"])
        self.assertEqual(config["section1"]["result"], "1")

    @aiotest
    def test_parse_custom_config(self):
        """
        this test should parse a custom config file
        """
        configfile = os.path.join(
            TEST_DIR, "resources", "test-1.conf")
        config = yield from parse_config(configfile)
        self.assertEqual(config.sections(), ["foo", "bar"])
        self.assertEqual(config["foo"]["bar"], "1")
        self.assertEqual(config["bar"]["foo"], "baz")

    @aiotest
    def test_config_string(self):
        config = yield from parse_config(config_string=CONFIG_STRING)
        self.assertEqual(config.sections(), ["section1", "section2"])
        self.assertEqual(config["section1"]["foo"], "2")
        self.assertEqual(config["section2"]["bar"], "3")


class AioConfigFinderTestCase(unittest.TestCase):

    def test_find_config(self):
        """
        this test should find "aio.conf" in the app_dir
        """
        app_dir = os.path.join(
            TEST_DIR, "resources")
        self.assertEqual(
            find_config(app_dir),
            os.path.join(
                app_dir, 'aio.conf'))

    def test_find_config_etc(self):
        """
        this test should find "aio.conf" in the etc
        """
        app_dir = os.path.join(
            TEST_DIR, "resources", "sub")
        self.assertEqual(
            find_config(app_dir),
            os.path.join(
                app_dir, "etc", 'aio.conf'))



"""
aio.config
"""
import os
import sys
from setuptools import setup, find_packages

version = "0.0.7"

install_requires = ['distribute', 'aio.core']

if sys.version_info < (3, 4):
    install_requires += ['asyncio']

tests_require = install_requires + ['aio.testing']

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = read("README.rst")

setup(
    name='aio.config',
    version=version,
    description="Aio configuration utilities",
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    keywords='',
    author='Ryan Northey',
    author_email='ryan@3ca.org.uk',
    url='http://github.com/phlax/aio.config',
    license='GPL',
    packages=find_packages(),
    namespace_packages=['aio'],
    include_package_data=True,
    package_data={'': ['tests/resources/*.conf', 'tests/resources/sub/etc/*.conf', '*.rst']},    
    zip_safe=False,
    tests_require=tests_require,
    test_suite="aio.config.tests",
    install_requires=install_requires,
    entry_points="""
    # -*- Entry points: -*-
    """)
